@@ -23,7 +23,7 @@ public:
 public:
 	ColumnReader *GetChildReader(idx_t child_idx);
 
-	void InitializeRead(const std::vector<ColumnChunk> &columns, TProtocol &protocol_p) override;
+	void InitializeRead(idx_t row_group_idx_p, const std::vector<ColumnChunk> &columns, TProtocol &protocol_p) override;
 
 	idx_t Read(uint64_t num_values, parquet_filter_t &filter, uint8_t *define_out, uint8_t *repeat_out,
 	           Vector &result) override;
