@@ -28,6 +28,8 @@ static DefaultType internal_types[] = {{"int", LogicalTypeId::INTEGER},
                                        {"blob", LogicalTypeId::BLOB},
                                        {"varbinary", LogicalTypeId::BLOB},
                                        {"binary", LogicalTypeId::BLOB},
+                                       {"bit", LogicalTypeId::BIT},
+                                       {"bitstring", LogicalTypeId::BIT},
                                        {"int8", LogicalTypeId::BIGINT},
                                        {"bigint", LogicalTypeId::BIGINT},
                                        {"int64", LogicalTypeId::BIGINT},
@@ -78,7 +80,6 @@ static DefaultType internal_types[] = {{"int", LogicalTypeId::INTEGER},
                                        {"union", LogicalTypeId::UNION},
                                        {"timestamptz", LogicalTypeId::TIMESTAMP_TZ},
                                        {"timetz", LogicalTypeId::TIME_TZ},
-                                       {"json", LogicalTypeId::JSON},
                                        {"enum", LogicalTypeId::ENUM},
                                        {"null", LogicalTypeId::SQLNULL},
                                        {nullptr, LogicalTypeId::INVALID}};
