@@ -63,6 +63,10 @@ string PhysicalOperatorToString(PhysicalOperatorType type) {
 		return "IE_JOIN";
 	case PhysicalOperatorType::CROSS_PRODUCT:
 		return "CROSS_PRODUCT";
+	case PhysicalOperatorType::POSITIONAL_JOIN:
+		return "POSITIONAL_JOIN";
+	case PhysicalOperatorType::POSITIONAL_SCAN:
+		return "POSITIONAL_SCAN";
 	case PhysicalOperatorType::UNION:
 		return "UNION";
 	case PhysicalOperatorType::INSERT:
@@ -131,6 +135,8 @@ string PhysicalOperatorToString(PhysicalOperatorType type) {
 		return "ATTACH";
 	case PhysicalOperatorType::RESULT_COLLECTOR:
 		return "RESULT_COLLECTOR";
+	case PhysicalOperatorType::EXTENSION:
+		return "EXTENSION";
 	case PhysicalOperatorType::INVALID:
 		break;
 	}
