@@ -3,6 +3,7 @@
 #include "duckdb/planner/operator/logical_comparison_join.hpp"
 #include "duckdb/planner/operator/logical_create_index.hpp"
 #include "duckdb/planner/operator/logical_delim_join.hpp"
+#include "duckdb/planner/operator/logical_insert.hpp"
 
 #include "duckdb/planner/expression/bound_columnref_expression.hpp"
 #include "duckdb/planner/expression/bound_reference_expression.hpp"
@@ -51,7 +52,7 @@ void ColumnBindingResolver::VisitOperator(LogicalOperator &op) {
 		// CREATE INDEX statement, add the columns of the table with table index 0 to the binding set
 		// afterwards bind the expressions of the CREATE INDEX statement
 		auto &create_index = (LogicalCreateIndex &)op;
-		bindings = LogicalOperator::GenerateColumnBindings(0, create_index.table.columns.LogicalColumnCount());
+		bindings = LogicalOperator::GenerateColumnBindings(0, create_index.table.GetColumns().LogicalColumnCount());
 		VisitOperatorExpressions(op);
 		return;
 	} else if (op.type == LogicalOperatorType::LOGICAL_GET) {
@@ -59,6 +60,25 @@ void ColumnBindingResolver::VisitOperator(LogicalOperator &op) {
 		bindings = op.GetColumnBindings();
 		VisitOperatorExpressions(op);
 		return;
+	} else if (op.type == LogicalOperatorType::LOGICAL_INSERT) {
+		//! We want to execute the normal path, but also add a dummy 'excluded' binding if there is a
+		// ON CONFLICT DO UPDATE clause
+		auto &insert_op = (LogicalInsert &)op;
+		if (insert_op.action_type != OnConflictAction::THROW) {
+			VisitOperatorChildren(op);
+			auto dummy_bindings = LogicalOperator::GenerateColumnBindings(
+			    insert_op.excluded_table_index, insert_op.table->GetColumns().PhysicalColumnCount());
+			bindings.insert(bindings.begin(), dummy_bindings.begin(), dummy_bindings.end());
+			if (insert_op.on_conflict_condition) {
+				VisitExpression(&insert_op.on_conflict_condition);
+			}
+			if (insert_op.do_update_condition) {
+				VisitExpression(&insert_op.do_update_condition);
+			}
+			VisitOperatorExpressions(op);
+			bindings = op.GetColumnBindings();
+			return;
+		}
 	}
 	// general case
 	// first visit the children of this operator
