@@ -1,5 +1,6 @@
 #include "duckdb/execution/index/art/node16.hpp"
 
+#include "duckdb/execution/index/art/art.hpp"
 #include "duckdb/execution/index/art/node4.hpp"
 #include "duckdb/execution/index/art/node48.hpp"
 
@@ -11,6 +12,13 @@ Node16::Node16() : Node(NodeType::N16) {
 	memset(key, 16, sizeof(key));
 }
 
+idx_t Node16::MemorySize(ART &art, const bool &recurse) {
+	if (recurse) {
+		return prefix.MemorySize() + sizeof(*this) + RecursiveMemorySize(art);
+	}
+	return prefix.MemorySize() + sizeof(*this);
+}
+
 idx_t Node16::GetChildPos(uint8_t k) {
 	for (idx_t pos = 0; pos < count; pos++) {
 		if (key[pos] == k) {
@@ -69,12 +77,16 @@ void Node16::ReplaceChildPointer(idx_t pos, Node *node) {
 	children[pos] = node;
 }
 
-void Node16::InsertChild(Node *&node, uint8_t key_byte, Node *new_child) {
+bool Node16::ChildIsInMemory(idx_t pos) {
+	return children[pos] && !children[pos].IsSwizzled();
+}
+
+void Node16::InsertChild(ART &art, Node *&node, uint8_t key_byte, Node *new_child) {
 	Node16 *n = (Node16 *)node;
 
-	// Insert new child node into node
-	if (n->count < 16) {
-		// Insert element
+	// insert new child node into node
+	if (n->count < Node16::GetSize()) {
+		// still space, just insert the child
 		idx_t pos = 0;
 		while (pos < node->count && n->key[pos] < key_byte) {
 			pos++;
@@ -88,57 +100,74 @@ void Node16::InsertChild(Node *&node, uint8_t key_byte, Node *new_child) {
 		n->key[pos] = key_byte;
 		n->children[pos] = new_child;
 		n->count++;
+
 	} else {
-		// Grow to Node48
+		// node is full, grow to Node48
 		auto new_node = Node48::New();
+		art.memory_size += new_node->MemorySize(art, false);
+		new_node->count = node->count;
+		new_node->prefix = std::move(n->prefix);
+
 		for (idx_t i = 0; i < node->count; i++) {
 			new_node->child_index[n->key[i]] = i;
 			new_node->children[i] = n->children[i];
 			n->children[i] = nullptr;
 		}
-		new_node->prefix = std::move(n->prefix);
-		new_node->count = node->count;
+
+		D_ASSERT(art.memory_size >= node->MemorySize(art, false));
+		art.memory_size -= node->MemorySize(art, false);
 		Node::Delete(node);
 		node = new_node;
-
-		Node48::InsertChild(node, key_byte, new_child);
+		Node48::InsertChild(art, node, key_byte, new_child);
 	}
 }
 
-void Node16::EraseChild(Node *&node, int pos, ART &art) {
+void Node16::EraseChild(ART &art, Node *&node, idx_t pos) {
+
 	auto n = (Node16 *)node;
+	D_ASSERT(pos < n->count);
+
+	// adjust the ART size
+	if (n->ChildIsInMemory(pos)) {
+		auto child = n->GetChild(art, pos);
+		D_ASSERT(art.memory_size >= child->MemorySize(art, true));
+		art.memory_size -= child->MemorySize(art, true);
+	}
+
 	// erase the child and decrease the count
 	n->children[pos].Reset();
 	n->count--;
+
 	// potentially move any children backwards
 	for (; pos < n->count; pos++) {
 		n->key[pos] = n->key[pos + 1];
 		n->children[pos] = n->children[pos + 1];
 	}
 	// set any remaining nodes as nullptr
-	for (; pos < 16; pos++) {
+	for (; pos < Node16::GetSize(); pos++) {
 		if (!n->children[pos]) {
 			break;
 		}
 		n->children[pos] = nullptr;
 	}
 
-	if (node->count <= 3) {
-		// Shrink node
+	// shrink node to Node4
+	if (node->count < Node4::GetSize()) {
+
 		auto new_node = Node4::New();
-		for (unsigned i = 0; i < n->count; i++) {
+		art.memory_size += new_node->MemorySize(art, false);
+		new_node->prefix = std::move(n->prefix);
+
+		for (idx_t i = 0; i < n->count; i++) {
 			new_node->key[new_node->count] = n->key[i];
 			new_node->children[new_node->count++] = n->children[i];
 			n->children[i] = nullptr;
 		}
-		new_node->prefix = std::move(n->prefix);
+
+		D_ASSERT(art.memory_size >= node->MemorySize(art, false));
+		art.memory_size -= node->MemorySize(art, false);
 		Node::Delete(node);
 		node = new_node;
 	}
 }
-
-idx_t Node16::GetSize() {
-	return 16;
-}
-
 } // namespace duckdb
