@@ -6,17 +6,18 @@
 
 namespace duckdb {
 
-void PhysicalSet::SetExtensionVariable(ExecutionContext &context, DBConfig &config,
-                                       ExtensionOption &extension_option) const {
+void PhysicalSet::SetExtensionVariable(ClientContext &context, ExtensionOption &extension_option, const string &name,
+                                       SetScope scope, const Value &value) {
+	auto &config = DBConfig::GetConfig(context);
 	auto &target_type = extension_option.type;
-	Value target_value = value.CastAs(context.client, target_type);
+	Value target_value = value.CastAs(context, target_type);
 	if (extension_option.set_function) {
-		extension_option.set_function(context.client, scope, target_value);
+		extension_option.set_function(context, scope, target_value);
 	}
 	if (scope == SetScope::GLOBAL) {
 		config.SetOption(name, std::move(target_value));
 	} else {
-		auto &client_config = ClientConfig::GetConfig(context.client);
+		auto &client_config = ClientConfig::GetConfig(context);
 		client_config.set_variables[name] = std::move(target_value);
 	}
 }
@@ -39,7 +40,7 @@ void PhysicalSet::GetData(ExecutionContext &context, DataChunk &chunk, GlobalSou
 			throw CatalogException("unrecognized configuration parameter \"%s\"\n%s", name,
 			                       StringUtil::CandidatesErrorMessage(potential_names, name, "Did you mean"));
 		}
-		SetExtensionVariable(context, config, entry->second);
+		SetExtensionVariable(context.client, entry->second, name, scope, value);
 		return;
 	}
 	SetScope variable_scope = scope;
