@@ -29,8 +29,9 @@ string PhysicalComparisonJoin::ParamsToString() const {
 		string op = ExpressionTypeToOperator(it.comparison);
 		extra_info += it.left->GetName() + " " + op + " " + it.right->GetName() + "\n";
 	}
-	extra_info += "\nEC = " + std::to_string(estimated_props->GetCardinality<double>()) + "\n";
-	extra_info += "COST = " + std::to_string(estimated_props->GetCost()) + "\n";
+	extra_info += "\n[INFOSEPARATOR]\n";
+	extra_info += StringUtil::Format("EC: %llu\n", estimated_props->GetCardinality<idx_t>());
+	extra_info += StringUtil::Format("Cost: %llu", (idx_t)estimated_props->GetCost());
 	return extra_info;
 }
 
