@@ -1,12 +1,27 @@
 #include "duckdb/execution/operator/schema/physical_create_index.hpp"
 
-#include "duckdb/catalog/catalog_entry/index_catalog_entry.hpp"
+#include "duckdb/catalog/catalog_entry/duck_table_entry.hpp"
+#include "duckdb/catalog/catalog_entry/table_catalog_entry.hpp"
+#include "duckdb/catalog/catalog_entry/duck_index_entry.hpp"
 #include "duckdb/main/client_context.hpp"
 #include "duckdb/storage/storage_manager.hpp"
 #include "duckdb/main/database_manager.hpp"
 
 namespace duckdb {
 
+PhysicalCreateIndex::PhysicalCreateIndex(LogicalOperator &op, TableCatalogEntry &table_p,
+                                         const vector<column_t> &column_ids, unique_ptr<CreateIndexInfo> info,
+                                         vector<unique_ptr<Expression>> unbound_expressions,
+                                         idx_t estimated_cardinality)
+    : PhysicalOperator(PhysicalOperatorType::CREATE_INDEX, op.types, estimated_cardinality),
+      table((DuckTableEntry &)table_p), info(std::move(info)), unbound_expressions(std::move(unbound_expressions)) {
+	D_ASSERT(table_p.IsDuckTable());
+	// convert virtual column ids to storage column ids
+	for (auto &column_id : column_ids) {
+		storage_ids.push_back(table.GetColumns().LogicalToPhysical(LogicalIndex(column_id)).index);
+	}
+}
+
 //===--------------------------------------------------------------------===//
 // Sink
 //===--------------------------------------------------------------------===//
@@ -34,8 +49,9 @@ unique_ptr<GlobalSinkState> PhysicalCreateIndex::GetGlobalSinkState(ClientContex
 	// create the global index
 	switch (info->index_type) {
 	case IndexType::ART: {
-		state->global_index = make_unique<ART>(storage_ids, TableIOManager::Get(*table.storage), unbound_expressions,
-		                                       info->constraint_type, table.storage->db);
+		auto &storage = table.GetStorage();
+		state->global_index = make_unique<ART>(storage_ids, TableIOManager::Get(storage), unbound_expressions,
+		                                       info->constraint_type, storage.db, true);
 		break;
 	}
 	default:
@@ -45,14 +61,14 @@ unique_ptr<GlobalSinkState> PhysicalCreateIndex::GetGlobalSinkState(ClientContex
 }
 
 unique_ptr<LocalSinkState> PhysicalCreateIndex::GetLocalSinkState(ExecutionContext &context) const {
-
 	auto state = make_unique<CreateIndexLocalSinkState>(context.client);
 
 	// create the local index
 	switch (info->index_type) {
 	case IndexType::ART: {
-		state->local_index = make_unique<ART>(storage_ids, TableIOManager::Get(*table.storage), unbound_expressions,
-		                                      info->constraint_type, table.storage->db);
+		auto &storage = table.GetStorage();
+		state->local_index = make_unique<ART>(storage_ids, TableIOManager::Get(storage), unbound_expressions,
+		                                      info->constraint_type, storage.db, false);
 		break;
 	}
 	default:
@@ -79,18 +95,17 @@ SinkResultType PhysicalCreateIndex::Sink(ExecutionContext &context, GlobalSinkSt
 	lstate.arena_allocator.Reset();
 	ART::GenerateKeys(lstate.arena_allocator, lstate.key_chunk, lstate.keys);
 
+	auto &storage = table.GetStorage();
 	auto art = make_unique<ART>(lstate.local_index->column_ids, lstate.local_index->table_io_manager,
 	                            lstate.local_index->unbound_expressions, lstate.local_index->constraint_type,
-	                            table.storage->db);
-	art->ConstructFromSorted(lstate.key_chunk.size(), lstate.keys, row_identifiers);
+	                            storage.db, false);
+	if (!art->ConstructFromSorted(lstate.key_chunk.size(), lstate.keys, row_identifiers)) {
+		throw ConstraintException("Data contains duplicates on indexed column(s)");
+	}
 
 	// merge into the local ART
-	{
-		IndexLock local_lock;
-		lstate.local_index->InitializeLock(local_lock);
-		if (!lstate.local_index->MergeIndexes(local_lock, art.get())) {
-			throw ConstraintException("Data contains duplicates on indexed column(s)");
-		}
+	if (!lstate.local_index->MergeIndexes(art.get())) {
+		throw ConstraintException("Data contains duplicates on indexed column(s)");
 	}
 	return SinkResultType::NEED_MORE_INPUT;
 }
@@ -102,7 +117,9 @@ void PhysicalCreateIndex::Combine(ExecutionContext &context, GlobalSinkState &gs
 	auto &lstate = (CreateIndexLocalSinkState &)lstate_p;
 
 	// merge the local index into the global index
-	gstate.global_index->MergeIndexes(lstate.local_index.get());
+	if (!gstate.global_index->MergeIndexes(lstate.local_index.get())) {
+		throw ConstraintException("Data contains duplicates on indexed column(s)");
+	}
 }
 
 SinkFinalizeType PhysicalCreateIndex::Finalize(Pipeline &pipeline, Event &event, ClientContext &context,
@@ -111,25 +128,30 @@ SinkFinalizeType PhysicalCreateIndex::Finalize(Pipeline &pipeline, Event &event,
 	// here, we just set the resulting global index as the newly created index of the table
 
 	auto &state = (CreateIndexGlobalSinkState &)gstate_p;
-
-	if (!table.storage->IsRoot()) {
+	auto &storage = table.GetStorage();
+	if (!storage.IsRoot()) {
 		throw TransactionException("Transaction conflict: cannot add an index to a table that has been altered!");
 	}
 
+	state.global_index->Verify();
+	if (state.global_index->track_memory) {
+		state.global_index->buffer_manager.IncreaseUsedMemory(state.global_index->memory_size);
+	}
+
 	auto &schema = *table.schema;
-	auto index_entry = (IndexCatalogEntry *)schema.CreateIndex(context, info.get(), &table);
+	auto index_entry = (DuckIndexEntry *)schema.CreateIndex(context, info.get(), &table);
 	if (!index_entry) {
 		// index already exists, but error ignored because of IF NOT EXISTS
 		return SinkFinalizeType::READY;
 	}
 
 	index_entry->index = state.global_index.get();
-	index_entry->info = table.storage->info;
+	index_entry->info = storage.info;
 	for (auto &parsed_expr : info->parsed_expressions) {
 		index_entry->parsed_expressions.push_back(parsed_expr->Copy());
 	}
 
-	table.storage->info->indexes.AddIndex(std::move(state.global_index));
+	storage.info->indexes.AddIndex(std::move(state.global_index));
 	return SinkFinalizeType::READY;
 }
 
