@@ -123,7 +123,7 @@ unique_ptr<PhysicalOperator> PhysicalPlanGenerator::CreatePlan(LogicalAggregate
 
 	auto plan = CreatePlan(*op.children[0]);
 
-	plan = ExtractAggregateExpressions(move(plan), op.expressions, op.groups);
+	plan = ExtractAggregateExpressions(std::move(plan), op.expressions, op.groups);
 
 	if (op.groups.empty()) {
 		// no groups, check if we can use a simple aggregation
@@ -138,11 +138,11 @@ unique_ptr<PhysicalOperator> PhysicalPlanGenerator::CreatePlan(LogicalAggregate
 			}
 		}
 		if (use_simple_aggregation) {
-			groupby = make_unique_base<PhysicalOperator, PhysicalUngroupedAggregate>(op.types, move(op.expressions),
-			                                                                         op.estimated_cardinality);
+			groupby = make_unique_base<PhysicalOperator, PhysicalUngroupedAggregate>(
+			    op.types, std::move(op.expressions), op.estimated_cardinality);
 		} else {
-			groupby = make_unique_base<PhysicalOperator, PhysicalHashAggregate>(context, op.types, move(op.expressions),
-			                                                                    op.estimated_cardinality);
+			groupby = make_unique_base<PhysicalOperator, PhysicalHashAggregate>(
+			    context, op.types, std::move(op.expressions), op.estimated_cardinality);
 		}
 	} else {
 		// groups! create a GROUP BY aggregator
@@ -150,15 +150,15 @@ unique_ptr<PhysicalOperator> PhysicalPlanGenerator::CreatePlan(LogicalAggregate
 		vector<idx_t> required_bits;
 		if (CanUsePerfectHashAggregate(context, op, required_bits)) {
 			groupby = make_unique_base<PhysicalOperator, PhysicalPerfectHashAggregate>(
-			    context, op.types, move(op.expressions), move(op.groups), move(op.group_stats), move(required_bits),
-			    op.estimated_cardinality);
+			    context, op.types, std::move(op.expressions), std::move(op.groups), std::move(op.group_stats),
+			    std::move(required_bits), op.estimated_cardinality);
 		} else {
 			groupby = make_unique_base<PhysicalOperator, PhysicalHashAggregate>(
-			    context, op.types, move(op.expressions), move(op.groups), move(op.grouping_sets),
-			    move(op.grouping_functions), op.estimated_cardinality);
+			    context, op.types, std::move(op.expressions), std::move(op.groups), std::move(op.grouping_sets),
+			    std::move(op.grouping_functions), op.estimated_cardinality);
 		}
 	}
-	groupby->children.push_back(move(plan));
+	groupby->children.push_back(std::move(plan));
 	return groupby;
 }
 
@@ -172,8 +172,8 @@ PhysicalPlanGenerator::ExtractAggregateExpressions(unique_ptr<PhysicalOperator>
 	for (auto &group : groups) {
 		auto ref = make_unique<BoundReferenceExpression>(group->return_type, expressions.size());
 		types.push_back(group->return_type);
-		expressions.push_back(move(group));
-		group = move(ref);
+		expressions.push_back(std::move(group));
+		group = std::move(ref);
 	}
 
 	for (auto &aggr : aggregates) {
@@ -181,23 +181,24 @@ PhysicalPlanGenerator::ExtractAggregateExpressions(unique_ptr<PhysicalOperator>
 		for (auto &child : bound_aggr.children) {
 			auto ref = make_unique<BoundReferenceExpression>(child->return_type, expressions.size());
 			types.push_back(child->return_type);
-			expressions.push_back(move(child));
-			child = move(ref);
+			expressions.push_back(std::move(child));
+			child = std::move(ref);
 		}
 		if (bound_aggr.filter) {
 			auto &filter = bound_aggr.filter;
 			auto ref = make_unique<BoundReferenceExpression>(filter->return_type, expressions.size());
 			types.push_back(filter->return_type);
-			expressions.push_back(move(filter));
-			bound_aggr.filter = move(ref);
+			expressions.push_back(std::move(filter));
+			bound_aggr.filter = std::move(ref);
 		}
 	}
 	if (expressions.empty()) {
 		return child;
 	}
-	auto projection = make_unique<PhysicalProjection>(move(types), move(expressions), child->estimated_cardinality);
-	projection->children.push_back(move(child));
-	return move(projection);
+	auto projection =
+	    make_unique<PhysicalProjection>(std::move(types), std::move(expressions), child->estimated_cardinality);
+	projection->children.push_back(std::move(child));
+	return std::move(projection);
 }
 
 } // namespace duckdb
