@@ -27,7 +27,7 @@ unique_ptr<PhysicalOperator> PhysicalPlanGenerator::CreateDistinctOn(unique_ptr<
 			group_by_references[bound_ref.index] = i;
 		}
 		aggregate_types.push_back(target->return_type);
-		groups.push_back(move(target));
+		groups.push_back(std::move(target));
 	}
 	bool requires_projection = false;
 	if (types.size() != group_count) {
@@ -50,40 +50,41 @@ unique_ptr<PhysicalOperator> PhysicalPlanGenerator::CreateDistinctOn(unique_ptr<
 			// entry is not one of the groups: need to push a FIRST aggregate
 			auto bound = make_unique<BoundReferenceExpression>(logical_type, i);
 			vector<unique_ptr<Expression>> first_children;
-			first_children.push_back(move(bound));
+			first_children.push_back(std::move(bound));
 
 			FunctionBinder function_binder(context);
 			auto first_aggregate = function_binder.BindAggregateFunction(
-			    FirstFun::GetFunction(logical_type), move(first_children), nullptr, AggregateType::NON_DISTINCT);
+			    FirstFun::GetFunction(logical_type), std::move(first_children), nullptr, AggregateType::NON_DISTINCT);
 			// add the projection
 			projections.push_back(make_unique<BoundReferenceExpression>(logical_type, group_count + aggregates.size()));
 			// push it to the list of aggregates
 			aggregate_types.push_back(logical_type);
-			aggregates.push_back(move(first_aggregate));
+			aggregates.push_back(std::move(first_aggregate));
 			requires_projection = true;
 		}
 	}
 
-	child = ExtractAggregateExpressions(move(child), aggregates, groups);
+	child = ExtractAggregateExpressions(std::move(child), aggregates, groups);
 
 	// we add a physical hash aggregation in the plan to select the distinct groups
-	auto groupby = make_unique<PhysicalHashAggregate>(context, aggregate_types, move(aggregates), move(groups),
-	                                                  child->estimated_cardinality);
-	groupby->children.push_back(move(child));
+	auto groupby = make_unique<PhysicalHashAggregate>(context, aggregate_types, std::move(aggregates),
+	                                                  std::move(groups), child->estimated_cardinality);
+	groupby->children.push_back(std::move(child));
 	if (!requires_projection) {
-		return move(groupby);
+		return std::move(groupby);
 	}
 
 	// we add a physical projection on top of the aggregation to project all members in the select list
-	auto aggr_projection = make_unique<PhysicalProjection>(types, move(projections), groupby->estimated_cardinality);
-	aggr_projection->children.push_back(move(groupby));
-	return move(aggr_projection);
+	auto aggr_projection =
+	    make_unique<PhysicalProjection>(types, std::move(projections), groupby->estimated_cardinality);
+	aggr_projection->children.push_back(std::move(groupby));
+	return std::move(aggr_projection);
 }
 
 unique_ptr<PhysicalOperator> PhysicalPlanGenerator::CreatePlan(LogicalDistinct &op) {
 	D_ASSERT(op.children.size() == 1);
 	auto plan = CreatePlan(*op.children[0]);
-	return CreateDistinctOn(move(plan), move(op.distinct_targets));
+	return CreateDistinctOn(std::move(plan), std::move(op.distinct_targets));
 }
 
 } // namespace duckdb
