@@ -6,6 +6,7 @@
 #include "duckdb/main/config.hpp"
 #include "duckdb/execution/operator/persistent/physical_batch_insert.hpp"
 #include "duckdb/parallel/task_scheduler.hpp"
+#include "duckdb/catalog/duck_catalog.hpp"
 
 namespace duckdb {
 
@@ -44,6 +45,41 @@ bool PhysicalPlanGenerator::UseBatchIndex(PhysicalOperator &plan) {
 	return UseBatchIndex(context, plan);
 }
 
+unique_ptr<PhysicalOperator> DuckCatalog::PlanInsert(ClientContext &context, LogicalInsert &op,
+                                                     unique_ptr<PhysicalOperator> plan) {
+	bool parallel_streaming_insert = !PhysicalPlanGenerator::PreserveInsertionOrder(context, *plan);
+	bool use_batch_index = PhysicalPlanGenerator::UseBatchIndex(context, *plan);
+	auto num_threads = TaskScheduler::GetScheduler(context).NumberOfThreads();
+	if (op.return_chunk) {
+		// not supported for RETURNING (yet?)
+		parallel_streaming_insert = false;
+		use_batch_index = false;
+	}
+	if (op.action_type != OnConflictAction::THROW) {
+		// We don't support ON CONFLICT clause in batch insertion operation currently
+		use_batch_index = false;
+	}
+	if (op.action_type == OnConflictAction::UPDATE) {
+		// When we potentially need to perform updates, we have to check that row is not updated twice
+		// that currently needs to be done for every chunk, which would add a huge bottleneck to parallelized insertion
+		parallel_streaming_insert = false;
+	}
+	unique_ptr<PhysicalOperator> insert;
+	if (use_batch_index && !parallel_streaming_insert) {
+		insert = make_unique<PhysicalBatchInsert>(op.types, op.table, op.column_index_map, std::move(op.bound_defaults),
+		                                          op.estimated_cardinality);
+	} else {
+		insert = make_unique<PhysicalInsert>(
+		    op.types, op.table, op.column_index_map, std::move(op.bound_defaults), std::move(op.expressions),
+		    std::move(op.set_columns), std::move(op.set_types), op.estimated_cardinality, op.return_chunk,
+		    parallel_streaming_insert && num_threads > 1, op.action_type, std::move(op.on_conflict_condition),
+		    std::move(op.do_update_condition), std::move(op.on_conflict_filter), std::move(op.columns_to_fetch));
+	}
+	D_ASSERT(plan);
+	insert->children.push_back(std::move(plan));
+	return insert;
+}
+
 unique_ptr<PhysicalOperator> PhysicalPlanGenerator::CreatePlan(LogicalInsert &op) {
 	unique_ptr<PhysicalOperator> plan;
 	if (!op.children.empty()) {
@@ -51,28 +87,7 @@ unique_ptr<PhysicalOperator> PhysicalPlanGenerator::CreatePlan(LogicalInsert &op
 		plan = CreatePlan(*op.children[0]);
 	}
 	dependencies.AddDependency(op.table);
-
-	bool parallel_streaming_insert = !PreserveInsertionOrder(*plan);
-	bool use_batch_index = UseBatchIndex(*plan);
-	auto num_threads = TaskScheduler::GetScheduler(context).NumberOfThreads();
-	if (op.return_chunk) {
-		// not supported for RETURNING (yet?)
-		parallel_streaming_insert = false;
-		use_batch_index = false;
-	}
-	unique_ptr<PhysicalOperator> insert;
-	if (use_batch_index && !parallel_streaming_insert) {
-		insert = make_unique<PhysicalBatchInsert>(op.types, op.table, op.column_index_map, std::move(op.bound_defaults),
-		                                          op.estimated_cardinality);
-	} else {
-		insert = make_unique<PhysicalInsert>(op.types, op.table, op.column_index_map, std::move(op.bound_defaults),
-		                                     op.estimated_cardinality, op.return_chunk,
-		                                     parallel_streaming_insert && num_threads > 1);
-	}
-	if (plan) {
-		insert->children.push_back(std::move(plan));
-	}
-	return insert;
+	return op.table->catalog->PlanInsert(context, op, std::move(plan));
 }
 
 } // namespace duckdb
